@@ -97,6 +97,10 @@ class PageRender extends Visitor {
 
 				$this->visit('',$attribute);
 			}
+
+			// Sort our attribute values for display, if we are the custom template.
+			if ($this->template->getID() == 'none')
+				$this->template->sort();
 		}
 	}
 
@@ -283,7 +287,7 @@ class PageRender extends Visitor {
 						break;
 
 					default:
-						$vals[$i] = password_hash($passwordvalue,$enc);
+						$vals[$i] = password_hash_custom($passwordvalue,$enc);
 				}
 
 				$vals = array_unique($vals);
@@ -331,7 +335,17 @@ class PageRender extends Visitor {
 			if (DEBUGTMP) printf('<font size=-2>%s:<u>%s</u></font><br />',__METHOD__,'Choosing the DEFAULT template, no other template applicable');
 
 			# Since getTemplate() returns a default template if the one we want doesnt exist, we can return $templates->getID(), it should be the default.
-			return $template->getID();
+			if ($_SESSION[APPCONFIG]->getValue('appearance','disable_default_template') AND $this->getMode() == 'creation') {
+
+				system_message(array(
+					'title'=>_('No available templates'),
+					'body'=>_('There are no available active templates for this container.'),
+					'type'=>'warn'));
+
+				return 'invalid';
+
+			} else
+				return $template->getID();
 
 		# If there is only 1 defined template, and no default available, then that is our template.
 		} elseif ((count($templates->getTemplates($this->getMode(),$this->getModeContainer(),true)) == 1) && ! $this->haveDefaultTemplate()) {
@@ -365,7 +379,7 @@ class PageRender extends Visitor {
 		$href = sprintf('cmd.php?cmd=schema&server_id=%s&view=attributes&viewvalue=%s',
 			$this->getServerID(),$attribute->getName());
 
-		if (! $_SESSION[APPCONFIG]->getValue('appearance','show_schema_link'))
+		if (! $_SESSION[APPCONFIG]->getValue('appearance','show_schema_link') || !$_SESSION[APPCONFIG]->isCommandAvailable('script','schema'))
 			printf('%s',_($attribute->getFriendlyName()));
 
 		elseif ($attribute->getLDAPtype())
@@ -943,7 +957,7 @@ class PageRender extends Visitor {
 		if (trim($val))
 			$enc_type = get_enc_type($val);
 		else
-			$enc_type = $server->getValue('appearance','password_hash');
+			$enc_type = $server->getValue('appearance','password_hash_custom');
 
 		$obfuscate_password = obfuscate_password_display($enc_type);
 
@@ -968,7 +982,7 @@ class PageRender extends Visitor {
 		if (trim($val))
 			$enc_type = get_enc_type($val);
 		else
-			$enc_type = $server->getValue('appearance','password_hash');
+			$enc_type = $server->getValue('appearance','password_hash_custom');
 
 		echo '<table cellspacing="0" cellpadding="0"><tr><td valign="top">';
 
@@ -1056,6 +1070,11 @@ class PageRender extends Visitor {
 			} else {
 				echo '<table cellspacing="0" cellpadding="0" border="0">';
 
+				// For checkbox items, we need to render a blank entry, so that we detect an all-unselect situation
+				printf('<tr><td colspan="2"><input type="hidden" id="new_values_%s_%s" name="new_values[%s][]" value="%s"/></td></tr>',
+					htmlspecialchars($attribute->getName()),$j++,
+					htmlspecialchars($attribute->getName()),'');
+
 				foreach ($attribute->getSelection() as $value => $description) {
 					if (in_array($value,$vals))
 						$selected[$value] = true;
