@@ -48,7 +48,7 @@ class TemplateRender extends PageRender {
 
 		$this->page = get_request('page','REQUEST',false,1);
 
-		if ($this->template_id) {
+		if ($this->template_id AND $this->template_id != 'invalid') {
 			if (! $this->template)
 				parent::accept();
 
@@ -2371,6 +2371,9 @@ function deleteAttribute(attrName,friendlyName,i)
 	protected function drawIconObjectClassAttribute($attribute,$val) {
 		if (DEBUGTMP) printf('<font size=-2>%s</font><br />',__METHOD__);
 
+		if (! $_SESSION[APPCONFIG]->getValue('appearance','show_schema_link') || !$_SESSION[APPCONFIG]->isCommandAvailable('script','schema'))
+			return;
+
 		if (strlen($val) > 0) {
 			$href = sprintf('cmd.php?cmd=schema&server_id=%s&view=objectclasses&viewvalue=%s',
 				$this->getServerID(),$val);
@@ -2463,7 +2466,7 @@ function deleteAttribute(attrName,friendlyName,i)
 		if ($val = $attribute->getValue($i))
 			$default = get_enc_type($val);
 		else
-			$default = $this->getServer()->getValue('appearance','password_hash');
+			$default = $this->getServer()->getValue('appearance','password_hash_custom');
 
 		if (! $attribute->getPostValue())
 			printf('<input type="hidden" name="post_value[%s][]" value="%s" />',$attribute->getName(),$i);
