@@ -368,11 +368,11 @@ abstract class DS {
 					$userDN = preg_replace($regex, $replacement, $_SERVER['REMOTE_USER']);
 
 					$CACHE[$this->index][$method] = $this->login($userDN, '', $method);
-				}
+
 				# Otherwise, use the user name as is
-				else {
-					$CACHE[$this->index][$method] = $this->login($_SERVER['REMOTE_USER'], '', $method);
-				}
+				# For GSSAPI Authentication + mod_auth_kerb and Basic Authentication
+				} else
+					$CACHE[$this->index][$method] = $this->login(isset($_SERVER['REMOTE_USER']) ? $_SERVER['REMOTE_USER'] : '', '', $method);
 
 				break;
 
@@ -542,6 +542,10 @@ class Datastore {
 			'desc'=>'Whether this server is visible',
 			'default'=>true);
 
+		$this->default->server['hide_noaccess_base'] = array(
+			'desc'=>'If base DNs are not accessible, hide them instead of showing create',
+			'default'=>false);
+
 		# Authentication Information
 		$this->default->login['auth_type'] = array(
 			'desc'=>'Authentication Type',
