@@ -13,13 +13,10 @@
  * @subpackage DataStore
  */
 class ldap_pla extends ldap {
-	# Attributes that should be treated as MAY attributes, even though the scheme has them as MUST attributes.
-	private $force_may = array();
-
 	function __construct($index) {
 		parent::__construct($index);
 
-		$this->default->appearance['password_hash'] = array(
+		$this->default->appearance['password_hash_custom'] = array(
 			'desc'=>'Default HASH to use for passwords',
 			'default'=>'md5');
 
@@ -27,6 +24,10 @@ class ldap_pla extends ldap {
 			'desc'=>'Whether to show the "Create new Entry here" in the tree browser',
 			'default'=>true);
 
+		$this->default->appearance['open_tree'] = array(
+			'desc'=>'Whether to initially open each tree',
+			'default'=>false);
+
 		$this->default->login['fallback_dn'] = array(
 			'desc'=>'If the attribute base login fails, see if a DN was entered',
 			'default'=>false);
@@ -89,6 +90,11 @@ class ldap_pla extends ldap {
 				'*'
 			));
 
+		$this->default->server['force_may'] = array(
+			'desc'=>'Force server MUST attributes as MAY attributes',
+			'default'=>array(
+			));
+
 		# Settings for auto_number
 		$this->default->auto_number['enable'] = array(
 			'desc'=>'Enable the AUTO UID feature',
